from types import ModuleType
from typing import Any, Optional, Callable, Tuple, List, Union, Dict

from django.db import models
from rest_framework import serializers

from .base import FieldInspector, SerializerInspector
from ..openapi import _TYPE_any, _FORMAT_any

# Alias for optional imported function: inspect.signature
typing: Optional[ModuleType]
inspect_signature: Optional[Callable]

class InlineSerializerInspector(SerializerInspector):
    use_definitions: bool = ...
    def get_schema(self, serializer: Any): ...
    def add_manual_parameters(self, serializer: Any, parameters: Any): ...
    def get_request_parameters(self, serializer: Any, in_: Any): ...
    def get_property_name(self, field_name: Any): ...
    def get_parameter_name(self, field_name: Any): ...
    def get_serializer_ref_name(self, serializer: Any): ...
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class ReferencingSerializerInspector(InlineSerializerInspector):
    use_definitions: bool = ...

def get_queryset_field(queryset: Any, field_name: Any): ...
def get_model_field(model: Any, field_name: Any): ...
def get_queryset_from_view(view: Any, serializer: Optional[Any] = ...): ...
def get_parent_serializer(field: Any): ...
def get_related_model(model: Any, source: Any): ...

class RelatedFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

def find_regex(regex_field: Any): ...

numeric_fields: Any
limit_validators: Any

def find_limits(field: Any): ...
def decimal_field_type(field: Any): ...

_BasicType = Tuple[_TYPE_any, Union[_FORMAT_any, Callable[[serializers.Field], _FORMAT_any], None]]

model_field_to_basic_type: List[Tuple[models.Field, _BasicType]]
ip_format: Dict[str, _FORMAT_any]
serializer_field_to_basic_type: List[Tuple[serializers.Field, _BasicType]]
basic_type_info: List[Tuple[Union[models.Field, serializers.Field], _BasicType]]

def get_basic_type_info(field: Any): ...
def decimal_return_type(): ...
def get_origin_type(hint_class: Any): ...
def hint_class_issubclass(hint_class: Any, check_class: Any): ...

hinting_type_info: List[Tuple[Union[type, Tuple[type, ...]], _BasicType]]

def inspect_collection_hint_class(hint_class: Any): ...
def get_basic_type_info_from_hint(hint_class: Any): ...

class SerializerMethodFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class SimpleFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class ChoiceFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class FileFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class DictFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class HiddenFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class JSONFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class StringDefaultFieldInspector(FieldInspector):
    def field_to_swagger_object(self, field: Any, swagger_object_type: Any, use_references: Any, **kwargs: Any): ...

class CamelCaseJSONFilter(FieldInspector):
    def camelize_string(self, s: Any): ...
    def camelize_schema(self, schema: Any) -> None: ...
    def process_result(self, result: Any, method_name: Any, obj: Any, **kwargs: Any): ...
    def is_camel_case(self): ...

class RecursiveFieldInspector(FieldInspector): ...
